import sys

if sys.platform == "win32":
    PIPE_NAME: str

    def can_talk_to_agent() -> bool: ...

    class OpenSSHAgentConnection:
        def __init__(self) -> None: ...
        def send(self, data: bytes) -> int: ...
        def recv(self, n: int) -> bytes: ...
        def close(self) -> None: ...
